/*
* generic_hid.c
*
*  Created on: Apr 22, 2011
*      Author: Jan Axelson
*
* Demonstrates communicating with a device designed for use with a generic HID-class USB device.
* Sends and receives 2-byte reports.
* Requires: an attached HID-class device that supports 2-byte
* Input, Output, and Feature reports.
* The device firmware should respond to a received report by sending a report.
* Change VENDOR_ID and PRODUCT_ID to match your device's Vendor ID and Product ID.
* See Lvr.com/winusb.htm for example device firmware.
* This firmware is adapted from code provided by Xiaofan.
* Note: libusb error codes are negative numbers.

The application uses the libusb 1.0 API from libusb.org.
Compile the application with the -lusb-1.0 option. 
Use the -I option if needed to specify the path to the libusb.h header file. For example:
-I/usr/local/angstrom/arm/arm-angstrom-linux-gnueabi/usr/include/libusb-1.0 

*/

#include <errno.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "libusb-1.0/libusb.h"
#include <fcntl.h>
#include "INOGENIHID.h"
#include "HIDCommand.h"
#include "stdafx.h"

// Values for bmRequestType in the Setup transaction's Data packet.

static const int CONTROL_REQUEST_TYPE_IN = LIBUSB_ENDPOINT_IN | LIBUSB_REQUEST_TYPE_CLASS | LIBUSB_RECIPIENT_INTERFACE;
static const int CONTROL_REQUEST_TYPE_OUT = LIBUSB_ENDPOINT_OUT | LIBUSB_REQUEST_TYPE_CLASS | LIBUSB_RECIPIENT_INTERFACE/*LIBUSB_RECIPIENT_DEVICE*/;

// From the HID spec:

static const int HID_GET_REPORT = 0x01;
static const int HID_SET_REPORT = 0x09;
static const int HID_REPORT_TYPE_INPUT = 0x01;
static const int HID_REPORT_TYPE_OUTPUT = 0x02;
static const int HID_REPORT_TYPE_FEATURE = 0x03;

// With firmware support, transfers can be > the endpoint's max packet size.
#define NO_ERROR			0 


static const int INTERRUPT_OUT_ENDPOINT = 0x01;

static const int INTERFACE_NUMBER = 2;
static const int INTERFACE_NUMBER_2 = 4;
static const int TIMEOUT_MS = 5000;


struct libusb_device_handle *devh = NULL;
libusb_context *context = NULL;

int SendReport(unsigned char *Data);
int ReceiveReport(unsigned char *Data);

int Connect(void)
{

	libusb_device **list = NULL;
	static const int INOGENI_VENDOR_ID = 0x2997;
	static const int INOGENI_PRODUCT_ID = 0x0001;
	int device_ready = 0;
	int result;
	int rc = 0;
	size_t idx =0;
	ssize_t count = 0;
	struct inogeni_hid_command command;
	int i = 0;
	
	rc = libusb_init(&context);
	if(rc != 0)
	{
		printf("error\n");
		return rc;
	}

	count = libusb_get_device_list(context, &list);
	if(!(count > 0))
	{
		printf("error\n");
		return LIBUSB_ERROR_NO_DEVICE;
	}

	for (idx = 0; idx < count; ++idx) 
	{
		libusb_device *device = list[idx];
		
		struct libusb_device_descriptor desc = {0};
		rc = libusb_get_device_descriptor(device, &desc);
		if(rc != 0)
		{
			printf("error\n");
			return rc;
		}

		// printf("Vendor:Device = %04x:%04x\n", desc.idVendor, desc.idProduct);
		if (desc.idVendor == INOGENI_VENDOR_ID)
		{
			//printf("INOGENI device found!\n");
			devh = libusb_open_device_with_vid_pid(context, desc.idVendor, desc.idProduct);
			if (devh == NULL)
			{
				printf("Error opening the device\n");
				return -1;
			}
		}
	}

	// Change these as needed to match idVendor and idProduct in your device's device descriptor.
	if (devh != NULL)
	{
		// The HID has been detected.
		// Detach the hidusb driver from the HID to enable using libusb.

		libusb_detach_kernel_driver(devh, INTERFACE_NUMBER);
		{
			result = libusb_claim_interface(devh, INTERFACE_NUMBER);
			if (result >= 0)
			{
				device_ready = 1;
			}
			else
			{
				libusb_detach_kernel_driver(devh, INTERFACE_NUMBER_2);
				result = libusb_claim_interface(devh, INTERFACE_NUMBER_2);
				if (result >= 0)
				{
					device_ready = 1;
				}
				else
				{
					libusb_detach_kernel_driver(devh, INTERFACE_NUMBER_2);
					fprintf(stderr, "libusb_claim_interface error %d\n", result);
				}
			}
		}
	}
	else
	{
		fprintf(stderr, "No INOGENI device found!\n");
		return LIBUSB_ERROR_NO_DEVICE;
	}


	if (device_ready)
	{
		return 0;
	}
	
	return LIBUSB_ERROR_NO_DEVICE;
}


int Disconnect(void)
{		
	// Finished using the device.
	if (devh != NULL)
	{
		libusb_release_interface(devh, 0); 
		
		libusb_close(devh);
	}
	
	if(context != NULL)
	libusb_exit(context);
	
	return 0;
}

int SendReport(unsigned char *Data)
{
	int bytes_received;
	int bytes_sent;
	int i = 0;
	int result = 0;
	//char data_out[INOGENI_REPORT_LEN];

	// Send data to the device.
	bytes_sent = libusb_control_transfer(
	devh,
	CONTROL_REQUEST_TYPE_OUT ,
	HID_SET_REPORT,
	(HID_REPORT_TYPE_OUTPUT<<8)|0x00,
	INTERFACE_NUMBER,
	Data,
	INOGENI_REPORT_LEN,
	TIMEOUT_MS);


	if (bytes_sent >= 0)
	{
#ifdef DEBUG_DISPLAY_RX_TX
		printf("Output report data sent:\n");
		for(i = 0; i < 64; i++)
		{
			printf("%02x ",Data[i]);
		}
		printf("\n");
#endif //DEBUG_DISPLAY_RX_TX
	}
	else
	{
		fprintf(stderr, "Error sending Input report %d\n", result);
		return result;
	}

	return 0;
}


int ReceiveReport(unsigned char *Data)
{
	int bytes_received;
	int bytes_sent;
	int i = 0;
	int result = 0;
	

	// Request data from the device.
	bytes_received = libusb_control_transfer(
	devh,
	CONTROL_REQUEST_TYPE_IN ,
	HID_GET_REPORT,
	(HID_REPORT_TYPE_INPUT<<8)|0x00,
	INTERFACE_NUMBER,
	Data,
	INOGENI_REPORT_LEN,
	TIMEOUT_MS);

	if (bytes_received >= 0)
	{
		
#ifdef DEBUG_DISPLAY_RX_TX
		printf("Input report data received:\n");
		for(i = 0; i < bytes_received; i++)
		{
			printf("%02x ", Data[i]);
		}
		printf("\n");

#endif //DEBUG_DISPLAY_RX_TX
	}
	else
	{
		fprintf(stderr, "Error receiving Input report %d\n", result);
		return result;
	}
	

	return 0;
}


int exchangeReport(unsigned char *data_out, unsigned char *data_in)
{
	
	int result = SendReport(data_out);
	if (result == NO_ERROR)
	{
		result = ReceiveReport(data_in);
	}
	return result;
}
